@@ -22,6 +22,7 @@
 #include <pthread.h>
 
 #include "arch.h"
+#include "defines.h"
 #include "list.h"
 
 /* logging buffer size */
@@ -95,7 +96,7 @@ enum logReturns
  * @param ... the arguments for the printf format c-string
  */
 #define LOG_DEVEL(log_level, args...) \
-    log_message_with_location(__func__, __FILE__, __LINE__, log_level, args);
+    log_message_with_location(__func__, __FILE__, __LINE__, log_level, args)
 
 /**
  * @brief Logging macro for messages that are for a systeam administrator to
@@ -109,7 +110,7 @@ enum logReturns
  * @param ... the arguments for the printf format c-string
  */
 #define LOG(log_level, args...) \
-    log_message_with_location(__func__, __FILE__, __LINE__, log_level, args);
+    log_message_with_location(__func__, __FILE__, __LINE__, log_level, args)
 
 /**
  * @brief Logging macro for logging the contents of a byte array using a hex
@@ -124,12 +125,31 @@ enum logReturns
  * @param length, the length of the byte array to log
  */
 #define LOG_DEVEL_HEXDUMP(log_level, message, buffer, length)  \
-    log_hexdump_with_location(__func__, __FILE__, __LINE__, log_level, message, buffer, length);
+    log_hexdump_with_location(__func__, __FILE__, __LINE__, log_level, message, buffer, length)
+
+/**
+ * @brief Logging macro for logging the contents of a byte array using a hex
+ * dump format.
+ *
+ * @param log_level, the log level
+ * @param message, a message prefix for the hex dump. Note: no printf like
+ *          formatting is done to this message.
+ * @param buffer, a pointer to the byte array to log as a hex dump
+ * @param length, the length of the byte array to log
+ */
+#define LOG_HEXDUMP(log_level, message, buffer, length)  \
+    log_hexdump_with_location(__func__, __FILE__, __LINE__, log_level, message, buffer, length)
 
 #else
-#define LOG_DEVEL(log_level, args...)
-#define LOG(log_level, args...) log_message(log_level, args);
-#define LOG_DEVEL_HEXDUMP(log_level, message, buffer, length)
+#define LOG(log_level, args...) log_message(log_level, args)
+#define LOG_HEXDUMP(log_level, message, buffer, length)  \
+    log_hexdump(log_level, message, buffer, length)
+
+/* Since log_message() returns a value ensure that the elided versions of
+ * LOG_DEVEL and LOG_DEVEL_HEXDUMP also "fake" returning the success value
+ */
+#define LOG_DEVEL(log_level, args...) UNUSED_VAR(LOG_STARTUP_OK)
+#define LOG_DEVEL_HEXDUMP(log_level, message, buffer, length) UNUSED_VAR(LOG_STARTUP_OK)
 
 #endif
 
@@ -263,11 +283,15 @@ internal_log_location_overrides_level(const char *function_name,
  * This function initialize the log facilities according to the configuration
  * file, that is described by the in parameter.
  * @param iniFile
- * @param applicationName, the name that is used in the log for the running application
+ * @param applicationName the name that is used in the log for the running
+ *                        application
+ * @param dump_on_start Whether to dump the config on stdout before
+ *                      logging is started
  * @return LOG_STARTUP_OK on success
  */
 enum logReturns
-log_start(const char *iniFile, const char *applicationName);
+log_start(const char *iniFile, const char *applicationName,
+          bool_t dump_on_start);
 
 /**
  * An alternative log_start where the caller gives the params directly.
@@ -336,6 +360,12 @@ log_end(void);
 enum logReturns
 log_message(const enum logLevels lvl, const char *msg, ...) printflike(2, 3);
 
+enum logReturns
+log_hexdump(const enum logLevels log_level,
+            const char *msg,
+            const char *p,
+            int len);
+
 /**
  * the log function that all files use to log an event,
  * with the function name and file line.
