@@ -28,6 +28,7 @@
 #include "arch.h"
 #include "parse.h"
 #include "ssl_calls.h"
+#include "log.h"
 
 #define MAX_SBYTES 0
 
@@ -454,7 +455,7 @@ trans_force_read_s(struct trans *self, struct stream *in_s, int size)
     int rcvd;
 
     if (self->status != TRANS_STATUS_UP ||
-        size < 0 || !s_check_rem_out(in_s, size))
+            size < 0 || !s_check_rem_out(in_s, size))
     {
         return 1;
     }
@@ -635,7 +636,7 @@ trans_write_copy_s(struct trans *self, struct stream *out_s)
     if (self->si != 0)
     {
         if ((self->si->cur_source != XRDP_SOURCE_NONE) &&
-            (self->si->cur_source != self->my_source))
+                (self->si->cur_source != self->my_source))
         {
             self->si->source[self->si->cur_source] += size;
             wait_s->source = self->si->source + self->si->cur_source;
@@ -662,7 +663,7 @@ trans_write_copy_s(struct trans *self, struct stream *out_s)
 
 /*****************************************************************************/
 int
-trans_write_copy(struct trans* self)
+trans_write_copy(struct trans *self)
 {
     return trans_write_copy_s(self, self->out_s);
 }
@@ -824,7 +825,9 @@ trans_listen_address(struct trans *self, char *port, const char *address)
     {
         self->sck = g_tcp_socket();
         if (self->sck < 0)
+        {
             return 1;
+        }
 
         g_tcp_set_non_blocking(self->sck);
 
@@ -846,7 +849,9 @@ trans_listen_address(struct trans *self, char *port, const char *address)
 
         self->sck = g_tcp_local_socket();
         if (self->sck < 0)
+        {
             return 1;
+        }
 
         g_tcp_set_non_blocking(self->sck);
 
@@ -975,13 +980,13 @@ trans_set_tls_mode(struct trans *self, const char *key, const char *cert,
     self->tls = ssl_tls_create(self, key, cert);
     if (self->tls == NULL)
     {
-        g_writeln("trans_set_tls_mode: ssl_tls_create malloc error");
+        LOG(LOG_LEVEL_ERROR, "trans_set_tls_mode: ssl_tls_create malloc error");
         return 1;
     }
 
     if (ssl_tls_accept(self->tls, ssl_protocols, tls_ciphers) != 0)
     {
-        g_writeln("trans_set_tls_mode: ssl_tls_accept failed");
+        LOG(LOG_LEVEL_ERROR, "trans_set_tls_mode: ssl_tls_accept failed");
         return 1;
     }
 
