@@ -79,11 +79,16 @@ struct mod
     int (*mod_get_wait_objs)(struct mod *v, tbus *read_objs, int *rcount,
                              tbus *write_objs, int *wcount, int *timeout);
     int (*mod_check_wait_objs)(struct mod *v);
-    int (*mod_frame_ack)(struct mod* mod, int flags, int frame_id);
-    int (*mod_suppress_output)(struct mod* mod, int suppress,
-                             int left, int top, int right, int bottom);
-    tintptr mod_dumby[100 - 11]; /* align, 100 minus the number of mod
-                                    functions above */
+    int (*mod_frame_ack)(struct mod *mod, int flags, int frame_id);
+    int (*mod_suppress_output)(struct mod *mod, int suppress,
+                               int left, int top, int right, int bottom);
+    int (*mod_server_monitor_resize)(struct mod *mod,
+                               int width, int height);
+    int (*mod_server_monitor_full_invalidate)(struct mod *mod,
+                               int width, int height);
+    int (*mod_server_version_message)(struct mod *mod);
+    tintptr mod_dumby[100 - 14]; /* align, 100 minus the number of mod
+                                 functions above */
     /* server functions */
     int (*server_begin_update)(struct mod *v);
     int (*server_end_update)(struct mod *v);
@@ -157,27 +162,27 @@ struct mod
                                     int flags);
     int (*server_set_pointer_ex)(struct mod *mod, int x, int y, char *data,
                                  char *mask, int bpp);
-    int (*server_add_char_alpha)(struct mod* mod, int font, int character,
+    int (*server_add_char_alpha)(struct mod *mod, int font, int character,
                                  int offset, int baseline,
-                                 int width, int height, char* data);
-    int (*server_create_os_surface_bpp)(struct mod* v, int rdpindex,
+                                 int width, int height, char *data);
+    int (*server_create_os_surface_bpp)(struct mod *v, int rdpindex,
                                         int width, int height, int bpp);
-    int (*server_paint_rect_bpp)(struct mod* v, int x, int y, int cx, int cy,
-                                 char* data, int width, int height,
+    int (*server_paint_rect_bpp)(struct mod *v, int x, int y, int cx, int cy,
+                                 char *data, int width, int height,
                                  int srcx, int srcy, int bpp);
-    int (*server_composite)(struct mod* v, int srcidx, int srcformat,
-                            int srcwidth, int srcrepeat, int* srctransform,
+    int (*server_composite)(struct mod *v, int srcidx, int srcformat,
+                            int srcwidth, int srcrepeat, int *srctransform,
                             int mskflags, int mskidx, int mskformat,
                             int mskwidth, int mskrepeat, int op,
                             int srcx, int srcy, int mskx, int msky,
                             int dstx, int dsty, int width, int height,
                             int dstformat);
-    int (*server_paint_rects)(struct mod* v,
+    int (*server_paint_rects)(struct mod *v,
                               int num_drects, short *drects,
                               int num_crects, short *crects,
                               char *data, int width, int height,
                               int flags, int frame_id);
-    int (*server_session_info)(struct mod* v, const char *data,
+    int (*server_session_info)(struct mod *v, const char *data,
                                int data_bytes);
     tintptr server_dumby[100 - 44]; /* align, 100 minus the number of server
                                        functions above */
