@@ -61,7 +61,7 @@ static tbus g_thread_done_event = 0;
 
 struct config_chansrv *g_cfg = NULL;
 
-int g_display_num = 0;
+int g_display_num = -1;
 int g_cliprdr_chan_id = -1; /* cliprdr */
 int g_rdpsnd_chan_id = -1;  /* rdpsnd  */
 int g_rdpdr_chan_id = -1;   /* rdpdr   */
@@ -1557,65 +1557,6 @@ x_server_fatal_handler(void)
     exit(0);
 }
 
-/*****************************************************************************/
-static int
-get_display_num_from_display(char *display_text)
-{
-    int index;
-    int mode;
-    int host_index;
-    int disp_index;
-    int scre_index;
-    char host[256];
-    char disp[256];
-    char scre[256];
-
-    g_memset(host, 0, 256);
-    g_memset(disp, 0, 256);
-    g_memset(scre, 0, 256);
-
-    index = 0;
-    host_index = 0;
-    disp_index = 0;
-    scre_index = 0;
-    mode = 0;
-
-    while (display_text[index] != 0)
-    {
-        if (display_text[index] == ':')
-        {
-            mode = 1;
-        }
-        else if (display_text[index] == '.')
-        {
-            mode = 2;
-        }
-        else if (mode == 0)
-        {
-            host[host_index] = display_text[index];
-            host_index++;
-        }
-        else if (mode == 1)
-        {
-            disp[disp_index] = display_text[index];
-            disp_index++;
-        }
-        else if (mode == 2)
-        {
-            scre[scre_index] = display_text[index];
-            scre_index++;
-        }
-
-        index++;
-    }
-
-    host[host_index] = 0;
-    disp[disp_index] = 0;
-    scre[scre_index] = 0;
-    g_display_num = g_atoi(disp);
-    return 0;
-}
-
 /*****************************************************************************/
 int
 main_cleanup(void)
@@ -1728,7 +1669,7 @@ main(int argc, char **argv)
     char text[256];
     const char *config_path;
     char log_path[256];
-    char *display_text;
+    const char *display_text;
     char log_file[256];
     enum logReturns error;
     struct log_config *logconfig;
@@ -1743,6 +1684,21 @@ main(int argc, char **argv)
         return 1;
     }
 
+    display_text = g_getenv("DISPLAY");
+    if (display_text == NULL)
+    {
+        g_writeln("DISPLAY is not set");
+        main_cleanup();
+        return 1;
+    }
+    g_display_num = g_get_display_num_from_display(display_text);
+    if (g_display_num < 0)
+    {
+        g_writeln("Unable to get display from DISPLAY='%s'", display_text);
+        main_cleanup();
+        return 1;
+    }
+
     /*
      * The user is unable at present to override the sysadmin-provided
      * sesman.ini location */
@@ -1755,11 +1711,6 @@ main(int argc, char **argv)
     config_dump(g_cfg);
 
     pid = g_getpid();
-    display_text = g_getenv("DISPLAY");
-    if (display_text != NULL)
-    {
-        get_display_num_from_display(display_text);
-    }
 
     /* starting logging subsystem */
     g_snprintf(log_file, 255, "%s/xrdp-chansrv.%d.log", log_path, g_display_num);
@@ -1813,13 +1764,6 @@ main(int argc, char **argv)
 
     LOG_DEVEL(LOG_LEVEL_INFO, "main: DISPLAY env var set to %s", display_text);
 
-    if (g_display_num == 0)
-    {
-        LOG_DEVEL(LOG_LEVEL_ERROR, "main: error, display is zero");
-        main_cleanup();
-        return 1;
-    }
-
     LOG_DEVEL(LOG_LEVEL_INFO, "main: using DISPLAY %d", g_display_num);
     g_snprintf(text, 255, "xrdp_chansrv_%8.8x_main_term", pid);
     g_term_event = g_create_wait_obj(text);
