@@ -11,6 +11,8 @@
 #include <sys/un.h>
 #include <sys/stat.h>
 
+#include "string_calls.h"
+
 #define PCSC_API
 
 typedef unsigned char BYTE;
@@ -153,66 +155,6 @@ lhexdump(void *p, int len)
     }
 }
 
-/*****************************************************************************/
-static int
-get_display_num_from_display(const char *display_text)
-{
-    int rv;
-    int index;
-    int mode;
-    int host_index;
-    int disp_index;
-    int scre_index;
-    char host[256];
-    char disp[256];
-    char scre[256];
-
-    memset(host, 0, 256);
-    memset(disp, 0, 256);
-    memset(scre, 0, 256);
-
-    index = 0;
-    host_index = 0;
-    disp_index = 0;
-    scre_index = 0;
-    mode = 0;
-
-    while (display_text[index] != 0)
-    {
-        if (display_text[index] == ':')
-        {
-            mode = 1;
-        }
-        else if (display_text[index] == '.')
-        {
-            mode = 2;
-        }
-        else if (mode == 0)
-        {
-            host[host_index] = display_text[index];
-            host_index++;
-        }
-        else if (mode == 1)
-        {
-            disp[disp_index] = display_text[index];
-            disp_index++;
-        }
-        else if (mode == 2)
-        {
-            scre[scre_index] = display_text[index];
-            scre_index++;
-        }
-        index++;
-    }
-    host[host_index] = 0;
-    disp[disp_index] = 0;
-    scre[scre_index] = 0;
-    LLOGLN(10, ("get_display_num_from_display: host [%s] disp [%s] scre [%s]",
-                host, disp, scre));
-    rv = atoi(disp);
-    return rv;
-}
-
 /*****************************************************************************/
 static int
 connect_to_chansrv(void)
@@ -252,11 +194,11 @@ connect_to_chansrv(void)
         LLOGLN(0, ("connect_to_chansrv: error, home not set"));
         return 1;
     }
-    dis = get_display_num_from_display(xrdp_display);
-    if (dis < 10)
+    dis = g_get_display_num_from_display(xrdp_display);
+    if (dis < 0)
     {
-        /* DISPLAY must be > 9 */
-        LLOGLN(0, ("connect_to_chansrv: error, display not > 9 %d", dis));
+        LLOGLN(0, ("connect_to_chansrv: error, don't understand DISPLAY='%s'",
+               xrdp_display));
         return 1;
     }
     g_sck = socket(PF_LOCAL, SOCK_STREAM, 0);
