@@ -92,7 +92,7 @@ sig_sesman_reload_cfg(int sig)
     g_cfg = cfg;
 
     /* start again logging subsystem */
-    error = log_start(g_cfg->sesman_ini, "xrdp-sesman");
+    error = log_start(g_cfg->sesman_ini, "xrdp-sesman", 0);
 
     if (error != LOG_STARTUP_OK)
     {
